class pantheraClass:
    """ Panthera class """

    panthera = ""

    def __init__(self, panthera):
        """ Initialize object """
    
        self.panthera = panthera
        self.app = panthera
        self.main()
        
    def main(self):
        return None